var express = require("express");
var router = express.Router();

const config = require("config");
const Key = require("../app/Key");
const Order = require("../app/Order");
const Manual = require("../app/payment_processor/Manual");

router.use((req, res, next) => {
    let auth_token = req.get("Authorization");
    let authorized = false;
    if (auth_token) {
        auth_token = auth_token.replace(/^Bearer /, "");
        let required_token = config.get("app.api_token");
        if (required_token === auth_token) {
            authorized = true;
        }
    }
    if (!authorized) {
        res.status(401).json({
            code: 404,
            error: "You are not authorized for API usage"
        });
    } else {
        next();
    }
});

router.get("/key/:key", (req, res) => {
    Key.GET_KEY(req.params.key, false).then((key) => {
        res.json({
            key: key.get_key(),
            charge: key.get_charge(),
            expiration: key.get_expiration_date().format("YYYY-MM-DD HH:mm:ss"),
            charge_orders: key.get_charge_orders()
        });
    });
})

router.post("/key/:key/discharge", (req, res) => {
    let amount = req.body.amount;
    let discharged_amount = 0;
    Key.GET_KEY(req.params.key, true).then((key) => {
        discharged_amount = key.discharge_key(amount);
        return key.save();
    }).then(key => {
        res.json({
            key: key.get_key(),
            charge: key.get_charge(),
            discharged: discharged_amount
        });
    });
})

router.post("/key/:key/charge", (req, res) => {
    let amount = req.body.amount;
    let payment_processor = "manual"; // Only supported processor via API currently
    let note = req.body.note;

    /**
     * @type {Order}
     */
    let new_order = null;
    Order.CREATE_NEW_ORDER(
        amount,
        req.params.key,
        new Manual(note)
    ).then(order => {
        new_order = order;
        return new_order.captureOrder();
    }).then(() => new_order.chargeKey())
        .then(async () => {
            res.json({
                key: await new_order.getKeyFromOrderLink(),
                order_id: new_order.getOrderID(),
                charged: new_order.getAmount(),
            });
        });
});

router.use((req, res) => {
    res.status(404).json({
        code: 404,
        error: `API with route ${res.locals.baseDir}${req.originalUrl} not found`
    })
});

module.exports = router;